
%% Bivariate normal distribution: visualization
% In this first part the pdf of a bivariate normal distribution
% is plotted.

% specify parameters of distribution
mu1 = 0;
mu2 = 0;
sigma1 = 0.8;
sigma2 = 1.2;
rho = 0.4;

% specify grid for visualization
gr = -6:0.1:6; % vector containing one-dimensional grid points
[xGr yGr] = meshgrid(gr,gr); 
    % create grid matrices from one-dimensional grid
    % vector: xGr contains x-coordinates of grid, 
    % yGr contains y-coordinates

% calculate normal pdf values on two-dimensional grid
vals = biv_normalPdf([xGr(:) yGr(:)],mu1,sigma1,mu2,sigma2,rho); 
    % command (:) vectorizes matrix: 
    % take column by column and attach them below 
    % each other
vals = reshape(vals,size(xGr)); 
    % transform vector of pdf-values back
    % to matrix: inversion of (:) command

% plot pdf 
mesh(xGr,yGr,vals) % specify x, y and z-coordinates as matrices
colormap gray % define color of figure

%% Joint normal distribution: conditional distributions
% As shown in the lecture slides, conditional distributions of
% joint normal distributions are normally distributed as well.
% That is, given that we already know the realization of random
% variable Y, we can update our knowledge of X through the link
% between both variables. Depending on the value of rho,
% knowledge of X does convey more or less information about X.
% Large rho values allow more updating.

% choose given y-value
yGiven = -1; 

% find respective row in coordinate matrices
ind = find(gr>=-1,1);

% include in graphic
hold on;
plot3(xGr(ind,:),yGr(ind,:),vals(ind,:),'-r','LineWidth',2) 
% plot three-dimensional line


%% Compare conditional distributions
% In this part conditional distributions will be shown for
% several values of Y. As can be seen, with positive linear
% dependence between X and Y (rho>0), larger given values of Y
% will lead to conditional distributions shifted further to the
% right.

figure; % open figure window
yGiven = [-3 -1 0 1 2 4 6]; % values of Y taken as given

for ii=1:numel(yGiven) 
    % for each conditional value, show updated distribution

    yGiv = yGiven(ii); % get current given Y value
    
    % get parameters of conditional distribution of X
    mu_given_y = mu1 + sigma1/sigma2*rho*(yGiv-mu2);
    sigma_given_y = sigma1*sqrt(1-rho^2);
    
    % plot conditional pdf
    cla % clear axes: delete previous content
    plot(gr,normpdf(gr,mu_given_y,sigma_given_y),'-r')

    % include unconditional pdf for comparison
    hold on; % keep previous conditional pdf
    plot(gr,normpdf(gr,mu1,sigma1));

    % include legend
    legend('conditional','unconditional')

    shg % show current graphics
    pause % stop until user presses key
end
    

    %% Simulating bivariate sample data
% We now want to simulate data from our given bivariate normal
% distribution. The simulation algorithm for two-dimensional
% distributions can be described in general as:
%       1.) simulate first variable 
%       2.) take first variable as given, and determine 
%           conditional distribution
%     	3.) use conditional distribution to simulate second
%           variable

% specify
nSim = 400;

%  simulate values of Y
U = rand(nSim,1); % simulate uniformly distributed data
Y = norminv(U,mu2,sigma2); 
    % Monte Carlo simulation: apply inverse cdf to uniform sample

% with given Y, simulate X
U = rand(nSim,1); % get random seed for X
X = zeros(nSim,1); % preallocation
for ii=1:nSim 
    yGiv = Y(ii); % get current Y value
    
    % get conditional parameters given Y
    mu_given_y = mu1 + sigma1/sigma2*rho*(yGiv-mu2);
    sigma_given_y = sigma1*sqrt(1-rho^2);
    
    % apply inverse conditional distribution to X
    X(ii) = norminv(U(ii),mu_given_y,sigma_given_y);
end

% plot sample points
plot(X,Y,'.')
title('Simulated sample data of joint normal distribution')
    
%% Clayton copula simulation
% As copula functions are distribution functions, the same
% general form of the simulation algorithm can be applied, too.
% That is, first simulate the first variable U1. Then, take
% individual values of U1 as given, and derive the conditional
% distribution of variable U2. Application of the inverse
% distribution function leads to sample data of U2. However, in
% order to get a better imagination of the conditional
% distributions involved, we first will take a look at the copual
% pdf, with colorized conditional distribution, as well as
% at the conditional cdf.

% show clayton pdf with conditional value

figure('pos',[50 50 1200 600]); % open figure window

u1_giv_values = [0.2 0.4 0.6 0.8]; % specify conditional values
for ii=u1_giv_values
    u1_giv = ii; % get current conditional value
theta = 4; % specify Clayton copula parameter

% specify grid on unit interval for figures
prec = 1e-5; 
    % distance to boundaries: 0 and 1 can lead to 
    % infinity values of copula pdf
ugr = [prec 0.01:0.02:0.99 1-prec];
[uGr vGr] = meshgrid(ugr,ugr); % get U- and V-coordinate matrices

% get pdf values on grid
pdfVals = clayton_pdf(theta,[uGr(:) vGr(:)]);
pdfVals = reshape(pdfVals,size(uGr)); 
    % inversion of vectorization operation

% get vfun values on grid
vVals = clay_vfun(vGr(:),uGr(:),theta);
vVals = reshape(vVals,size(uGr)); 
    % inversion of vectorization operation

% get conditional pdf values on grid
u1_giv_vector = repmat(u1_giv,numel(ugr),1); 
    % vector with equal entries for U coordinates
cond_pdf = ...
    clayton_pdf(theta,[u1_giv_vector ugr(:)]); 
    % get conditional pdf values

% get conditional v-function values on one-dimensional grid
cond_vVals = clay_vfun(ugr(:),u1_giv,theta);

subplot(1,2,1); % two subplot in a row
cla % clear axes from previous graphic
mesh(uGr,vGr,pdfVals,'EdgeColor',[0.8 0.8 0.8]); 
    % plot pdf function with gray lines
hold on;
contour3(uGr,vGr,1.02*pdfVals,[1:1:50]); % include contour lines 
h = get(gca,'children'); % get handles to individual contour lines
h = findobj(h,'type','patch'); 
    % get handles to individual contour lines
set(h,'LineWidth',2) % increase line width of contour lines
xlabel('U');
ylabel('V');
title(['U1 given: ' num2str(u1_giv,2)]);
set(gca,'ZLim',[0 10])

% include conditional value
plot3(u1_giv_vector,ugr,cond_pdf,'-r','LineWidth',2);

% plot v-function
subplot(1,2,2); % into second subplot
cla % clear axis
mesh(uGr,vGr,vVals,'EdgeColor',[0.8 0.8 0.8]); 
    % plot function in gray
hold on;
contour3(uGr,vGr,1.02*vVals,[0.1:0.1:1]); % include contour lines
h = get(gca,'children'); % get handles to individual contour lines
h = findobj(h,'type','patch'); 
    % get handles to individual contour lines
set(h,'LineWidth',2) % increase line width
xlabel('U_1');
ylabel('U_2|U_1');
title('v-funcion');
set(gca,'ZLim',[0 1]) % set z-limits of graphics

% include conditional value
plot3(u1_giv_vector,ugr,cond_vVals,'-r','LineWidth',2);

pause % wait for user to press key to continue
end

%% Simulating from Clayton copula

% specify number of simulated data values
nSim = 5000;

% get U1 values
U1 = rand(nSim,1);

% get random seed for U2
W2 = rand(nSim,1);

% transform W2 according to given U1 value with inverse
% v-function
U2 = zeros(nSim,1); % preallocation
for ii=1:nSim
    U2(ii) = clay_vinv(W2(ii),U1(ii),theta);
end

% show joint sample together with marginal histograms
h = scatterhist(U1,U2);    
set(get(h(1),'children'),'Marker','.'); 
    % modify sign used for visualization of data points
axes(h(1)) % get joint sample axes
axis square % make joint sample axes square






