%% Sklar's theorem: creating new distributions
% In this script we want to see how new distributions can be
% created by linking a copula with marginal distributions. 
% Additionally, the script shall enable to visually examine how
% changes with respect to either copula parameter or margin's
% parameter change the overall distribution function.

figure('pos',[50 50 1200 400])

% boolean indicating if copula or margins are varying
copVarying = false;

% specify multiple copula parameters
copParams = [1.2 2 6 10];

% specify multiple marginal distribution parameters
margParams = [0.2 0.5 0.8 1.6 4 8];

% specify copula family
copName = 'Gumbel';

% specify copula parameter, if copula is chosen constant
copParamFix = 3;

% specify marginal distributions
marg1 = 'tlocationscale';
marg2 = 'tlocationscale';

% specify marginal parameters
params1 = [-0.4 1];
params2 = [0.1267 0.8134 4];

% specify varying marginal parameters
margVarying = [2 4 8 12];

% specify varying parameters
varParams = copParams;


for ii=varParams % for each copula parameter

    if(copVarying)
        copParam = ii;
        
        % specify margins as distribution objects
        PD1 = ProbDistUnivParam(marg1,params1);
        PD2 = ProbDistUnivParam(marg2,params2);
    else
        copParam = copParamFix;
        
        % specify margins as distribution objects
        switch marg1
            case 'exp'
                PD1 = ProbDistUnivParam(marg1,ii);
            case 'normal' 
                PD1 = ProbDistUnivParam(marg1,[-0.4 ii]);
            case 'tlocationscale'
                PD1 = ProbDistUnivParam(marg1,[0 1 ii]);
        end
                
        PD2 = ProbDistUnivParam(marg2,params2);
    end
        

% specify ugr
prec = 1e-5; % distance to boundaries
ugr = [prec 0.01:0.02:0.99 1-prec];
[uGr vGr] = meshgrid(ugr,ugr); % get U and V coordinate matrices

% calculate copula density values
vals = copulapdf(copName,[uGr(:) vGr(:)],copParam);
vals = reshape(vals,size(uGr)); % revert vectorization operation

% specify grid of joint distribution
gr = -5:0.2:5;
[xGr yGr] = meshgrid(gr,gr);

% transform to copula grid
ugr = cdf(PD1,gr);
vgr = cdf(PD2,gr);
[uGr2 vGr2] = meshgrid(ugr,vgr);

% evaluate copula density on grid
vals2 = copulapdf(copName,[uGr2(:) vGr2(:)],copParam);
vals2 = reshape(vals2,size(uGr2));

% evaluate marginal distributions
pdfVals1 = pdf(PD1,gr);
pdfVals2 = pdf(PD2,gr);

% create matrices with marginal pdf-values
pdfValsMatr1 = repmat(pdfVals1,numel(pdfVals2),1);
pdfValsMatr2 = repmat(pdfVals2',1,numel(pdfVals2));

% put all parts together
vals2 = vals2.*pdfValsMatr1.*pdfValsMatr2;

% plot copula pdf
h1 = subplot(2,6,[1 2 7 8]); % plot copula pdf
cla
mesh_gray(h1,uGr,vGr,vals,[1:1:50]);
% mesh(uGr,vGr,vals,'EdgeColor',[0.8 0.8 0.8]);
% hold on;
% contour3(uGr,vGr,1.02*vals,[1:1:50]);
% h = get(gca,'children');
% h = findobj(h,'type','patch');
% set(h,'LineWidth',2)
% colormap Gray
% caxis([10 12])
% xlabel(' ');
% ylabel(' ');
% title(' ');
set(gca,'ZLim',[0 20])
newPos = get(gca,'pos');
newPos(1) = newPos(1)-0.04;
set(gca,'pos',newPos);
set(gca,'View',[-70 30])
title(['\alpha= ' num2str(copParam)])

subplot(2,6,[5 6 11 12])% plot pdf
cla
mesh_gray(gca,xGr,yGr,vals2,[0.02:0.02:4]);
% mesh(xGr,yGr,vals2,'EdgeColor',[0.8 0.8 0.8]);
% hold on;
% contour3(xGr,yGr,1.02*vals2,[0.02:0.02:4]);
% h = get(gca,'children');
% h = findobj(h,'type','patch');
% set(h,'LineWidth',2)
% colormap Gray
% caxis([10 12])
% xlabel(' ');
% ylabel(' ');
% title(' ');
set(gca,'ZLim',[0 5])
axis tight
newPos = get(gca,'pos');
newPos(1) = newPos(1)+0.04;
set(gca,'pos',newPos);
set(gca,'View',[-70 30])

subplot(2,6,[3 4])
plot(gr,pdfVals1,'LineWidth',2);
grid on;

subplot(2,6,[9 10])
plot(gr,pdfVals2,'LineWidth',2);
grid on;

shg
pause
end