%% Market risk application%       

%% load data
load data/DAX_data

%% data visualization
% Plot price path of each DAX stock

figure('pos',[50 50 1200 600])
for ii=1:29
    asset = ii; % get current asset
    cla % clear current axes
    plot(DAX_dates,DAX_prices(2:end,asset))
    title(DAX_Syms{asset}) % ticker symbol as title
    datetick 'x' % label x-axes with dates
    shg % show current graphics
    pause(2) % pause 2 seconds 
end


%% Calculate logarithmic percentage returns for single asset
asset = 7; % determine asset
returnSeries = 100*log(DAX_prices(2:end,asset)) - ...
    log(DAX_prices(1:end-1,asset));
plot(DAX_dates,returnSeries)
datetick 'x'


%% Plot normed prices together with return series
% Let's take a look at the price series of all DAX stocks. 
% This time however, prices are normalized to begin at price 1.
% This way price series are better comparable. Furthermore,
% unusually large returns become obvious, and prices can be
% cleaned with respect to events like stock splits.

% simultaneously get percentage log-returns for all assets
DAX_returns_raw = 100*log(DAX_prices(2:end,:)) - ...
    log(DAX_prices(1:end-1,:));

figure('pos',[50 50 1200 800])
DAX_normed_prices = cumsum(DAX_returns);
DAX_raw_normed_prices = cumsum(DAX_returns_raw);

for ii=1:29
    asset = ii; % get current asset
    
    subplot(2,1,1) % show normed prices
    cla % clear current axes
    plot(repmat(DAX_dates(:,1),1,29),DAX_raw_normed_prices,...
        'Color',0.8*[1 1 1]) % plot all normed prices
    hold on;
    plot(DAX_dates,DAX_raw_normed_prices(:,asset),'-r'); 
       % highlight current asset
    datetick 'x' % label x-axes with dates
    grid on % include grid in graphics
    title(DAX_Syms{ii}) % label figure with current ticker symbol
    
    subplot(2,1,2) % show uncleaned  DAX returns
    cla % clear axes
    plot(DAX_dates,DAX_returns_raw(:,asset))
    datetick 'x'
    grid on
    
    shg % show current graphics
    pause % wait for user to press key
end

%% Modelling marginal distributions
% As a start, we now want to model each asset individually.
% Easiest way is to assume normally distributed assets.

% specify asset
asset = 2;

% get cleaned percentage returns
rets = 100*DAX_returns(:,asset);

% fit normal distribution
objFun = @(x)neg_normalLLH(rets,x); % specify objective function
lb = [-inf 0.0001]; % get lower bounds for parameter restrictions
ub = [inf inf];
paramsHat = fmincon(objFun,[0 1],[],[],[],[],lb,ub,[],opt); 
% parameter estimation: numerical optimization

% alternative: using built-in Matlab function
[muHat2 sigmaHat2] = normfit(rets);

%% Goodness-of-fit: histogram
% Now that a normal distribution has been fitted, we have to
% examine whether the model is adequate.

% plot histogram with fitted distribution
[nPoints loc] = hist(rets,20); % get histogram

% plot normalized bars
bar(loc,nPoints/(numel(rets)*(loc(2)-loc(1))),...
    'FaceColor',[0.7 0.7 1])
hold on;
xLimits = get(gca,'XLim'); % set x-axes boundaries

% include fitted normal distribution
gr = xLimits(1):0.1:xLimits(2); % specify grid
plot(gr,normalPdf(gr,paramsHat),'r','LineWidth',2) 
   % include distribution

%% Goodness-of-fit: qq-plot

% use own qq-plot function
norm_qqPl(rets,paramsHat)

%% Alternative model: fit Student's t-distribution

% fit Student's t-distribution to data
nuHat = tFit(rets);

%% Goodness-of-fit: histogram

% plot histogram
[nPoints loc] = hist(rets,30);

% plot normalized bars
bar(loc,nPoints/(numel(rets)*(loc(2)-loc(1))),...
    'FaceColor',[0.7 0.7 1])
hold on;
xLimits = get(gca,'XLim');

% include fitted Student's t-distribution
gr = xLimits(1):0.1:xLimits(2);
plot(gr,tpdf(gr,nuHat),'r','LineWidth',2)

%% Goodness-of-fit: qq-plot
% compare empirical quantiles with empirical quantiles of
% sample data simulated from benchmark distribution

% number of simulated data points
nSim = 100000;

% simulate data from both distribution
simVals1 = normrnd(paramsHat(1),paramsHat(2),nSim,1);
simVals2 = trnd(nuHat,nSim,1);

figure('pos',[50 50 1200 600])
subplot(1,2,1)
    qqplot(rets,simVals1)
subplot(1,2,2)
    qqplot(rets,simVals2)

    
%% VaR application
% Now we estimate for each model of the marginal distribution the 
% associated VaR. Both models are compared with respect to exceedance 
% frequencies afterwards.

% specify confidence level
confLev = 0.01;

% specify asset
asset = 5;

% get returns
rets = 100*DAX_returns(:,asset);

% estimate normal distribution parameters with built-in function
[muHat sigmaHat] = normfit(rets);

% estimate Student's t distribution: account for mean daily return
nuHat = tFit(rets-mean(rets));

% get associated VaR
norm_VaR = norminv(confLev,muHat,sigmaHat);
t_VaR = tinv(confLev,nuHat)+mean(rets);

% get exceedances for both cases
norm_ex = rets<norm_VaR;
t_ex = rets<t_VaR;

% get exceedance frequencies
norm_frequ = sum(norm_ex)/numel(rets);
t_frequ = sum(t_ex)/numel(rets);

% plot
figure('pos',[50 50 1200 600])
subplot(2,1,1)  
    plot(DAX_dates,rets,'.b')
    hold on;
    plot(DAX_dates(norm_ex),rets(norm_ex),'.r')
    datetick 'x'
    title(num2str(norm_frequ,4))
    yLimits = get(gca,'YLim');
    set(gca,'YLim',[yLimits(1) 0])
    
subplot(2,1,2)  
    plot(DAX_dates,rets,'.b')
    hold on;
    plot(DAX_dates(t_ex),rets(t_ex),'.r')
    datetick 'x'
    title(num2str(t_frequ,4))
    yLimits = get(gca,'YLim');
    set(gca,'YLim',[yLimits(1) 0])
    

%% Portfolio modelling
% After modelling the risk of individual assets, we now take a look
% a portfolios consisting of two assets. This involves modelling the
% dependence structure between both assets.

% specify portfolio components
asset1 = 1;
asset2 = 7;

% get associated returns
rets1 = 100*DAX_returns(:,asset1);
rets2 = 100*DAX_returns(:,asset2);

%% Fitting models
% globally fit model with Student's t-copula and margins
[paramsHat time] = global_tCop_fit(rets1,rets2);

% fit same model with pseudo maximum likelihood
[paramsHat2 time2] = pseudo_tCop_fit(rets1,rets2);

% fit Student's t-copula, with empirically modelled 
% marginal distributions

% transform returns to empirical quantiles
[sorted IX1] = sort(rets1);
ranks1 = (1:numel(rets1))'/(numel(rets1)+1);
U1(IX1) = ranks1;
U1 = U1';

% second asset
[sorted IX2] = sort(rets2);
ranks2 = (1:numel(rets1))'/(numel(rets1)+1);
U2(IX2) = ranks2;
U2 = U2';

% fit copula to empirical ranks
[rhoHat nuHat] = copulafit('t',[U1 U2]);

%% Evaluate risk measures: VaR and ES
% Now that we have a model for the bivariate distribution
% of both assets, we want to evaluate VaR and ES for a 
% portfolio with given weights.

% calculate portfolio returns for given weights
port_ret = log_port_ret([rets1 rets2],[0.5 0.5]);

% plot portfolio returns
plot(DAX_dates,port_ret,'.')
datetick 'x'

% get VaR and ES with historical simulation
VAR_port_HS = quantile(port_ret,0.01);
ES_port_HS = mean(port_ret(port_ret<VAR_port_HS));

%% Via  Monte Carlo for given model

% specify number of simulated data points
nSim = 10000;

% simulate values from copula
U = copularnd('t',rhoHat,nuHat,nSim);

% plot values
plot(U(:,1),U(:,2),'.')

% transform according to marginal distributions
X1 = ksdensity(rets1,U(:,1),'function','icdf');
X2 = ksdensity(rets2,U(:,2),'function','icdf');

% show scatterhist
h = scatterhist(X1,X2);
set(get(h(1),'children'),'Marker','.')

% get associated  portfolio returns
sim_pRets = log_port_ret([X1 X2],[0.5 0.5]);

% get Monte Carlo values of VaR and ES
VaR_hat = quantile(sim_pRets,0.01);
ES_hat = mean(sim_pRets(sim_pRets<VaR_hat));

%% Backtesting: portfolio returns

% get portfolio returns below VaR
exceedances = sim_pRets<VaR_hat;
exceedances = [X1(exceedances) X2(exceedances)];

% colorize exceedances
axes(h(1))
hold on;
plot(exceedances(:,1),exceedances(:,2),'.r')

% show portfolio returns with VaR
hist(sim_pRets,40)
hold on;
yLimits = get(gca,'YLim');

% include VaR
plot([VaR_hat VaR_hat],yLimits,'-r')
title(['VaR: ' num2str(VaR_hat,4) ', ES: ' num2str(ES_hat,4)])

%% Stress-testing:
stressDependency([rets1 rets2],[0.5 0.5],30)


%% Extension: Time-series
%       conduct tests for volatility clusters
%       autocorr functions
%       simulate from GARCH(1,1)
%       derive sigma series for given parameters
%       evaluate llh for given parameters
%       fit GARCH(1,1)
%       show sigma series
%       include ar(1) term
%       theory: larger ARMA(p,q) models
%       -> model selection theory
%       simulate from fitted model
%       conduct multi-period analysis


