function stressDependency(X,wts,claytonParam)
% stress-test with clayton copula
%
% Inputs:
%   X       nxd vector containing asset returns 
%   wts     1xd vector containing portfolio weights
%   claytonParam    scalar value: Clayton parameter
%
% Outputs:
%   


%% specify parameters
% get number of assets / observations
nObs = numel(X(:,1));
nAss = numel(wts);

% specify number of simulations
nSim = 10000;

%% estimate Student's t-copula
% preallocation for empirical quantiles
U = zeros(size(X));

% transform asset returns to ranks via empirical distribution
for ii=1:nAss
    U(:,ii) = get_empQuants(X(:,ii));
end

% estimate copula
[rhoHat nuHat] = copulafit('t',U);

%% derive VaR and ES from estimated model
% get real VaR and ES
Usim = copularnd('t',rhoHat,nuHat,nSim);

% transform to "absolute terms"
Xsim = zeros(nSim,nAss);
for ii=1:nAss
    Xsim(:,ii) = ksdensity(X(:,ii),Usim(:,ii),'function','icdf');
end

% get portfolio returns
pRetsSim = log_port_ret(Xsim,wts);

% get VaR
VaR_real = quantile(pRetsSim,0.01);
ES_real = mean(pRetsSim(pRetsSim<VaR_real));

%% derive VaR and ES from stressed model
% get real VaR and ES
Usim = copularnd('clayton',claytonParam,nSim);

% transform to "absolute terms"
Xsim = zeros(nSim,nAss);
for ii=1:nAss
    Xsim(:,ii) = ksdensity(X(:,ii),Usim(:,ii),'function','icdf');
end

% get portfolio returns
pRetsSim_stress = log_port_ret(Xsim,wts);

% get VaR
VaR_stress = quantile(pRetsSim_stress,0.01);
ES_stress = mean(pRetsSim_stress(pRetsSim_stress<VaR_stress));

%% Compare visually
figure('pos',[50 50 1200 600])
subplot(1,2,1);
hist(pRetsSim,30)
title(['VaR: ' num2str(VaR_real,3) ', ES:' num2str(ES_real,3)])

subplot(1,2,2);
hist(pRetsSim_stress,30)
title(['VaR: ' num2str(VaR_stress,3) ', ES:' num2str(ES_stress,3)])

end

